// ====================================================================
// Copyright (c) 1997, 1998 The Apache Group.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer. 
//
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in
//    the documentation and/or other materials provided with the
//    distribution.
//
// 3. All advertising materials mentioning features or use of this
//    software must display the following acknowledgment:
//    "This product includes software developed by the Apache Group
//    for use in the Apache HTTP server project (http://www.apache.org/)."
//
// 4. The names "Apache Server" and "Apache Group" must not be used to
//    endorse or promote products derived from this software without
//    prior written permission.
//
// 5. Redistributions of any form whatsoever must retain the following
//    acknowledgment:
//    "This product includes software developed by the Apache Group
//    for use in the Apache HTTP server project (http://www.apache.org/)."
//
// THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
// EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
// ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
// NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
// ====================================================================
//
// This software consists of voluntary contributions made by many
// individuals on behalf of the Apache Group and was originally based
// on public domain software written at the National Center for
// Supercomputing Applications, University of Illinois, Urbana-Champaign.
// For more information on the Apache Group and the Apache HTTP server
// project, please see <http://www.apache.org/>.

// JServ - Serve up Java servlets
// by Alexei Kosut <akosut@apache.org>

// Parts are based on examples from  _Java in a Nutshell_ by David Flanagan:
// Written by David Flanagan.  Copyright (c) 1996 O'Reilly & Associates.
// You may study, use, modify, and distribute this example for any purpose.
// This example is provided WITHOUT WARRANTY either expressed or implied.

// ServletMonitor.java:
// - org.apache.jserv.test.ServletMonitor

package gs.admin;

import java.util.Enumeration;
import java.util.Hashtable;

import java.io.IOException;

import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletContext;
import javax.servlet.ServletConfig;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpUtils;

import java.sql.ResultSet;

import gs.db.update;
import gs.db.user;
import gs.db.groupusers;
import gs.db.RecordHandler;

import gs.html.Html;
import gs.html.Parser;

/**
 */
public class users extends HttpServlet {

    /**
     * Service the request.
     *
     * @exception  ServletException if there is any sort of Exception
     *
     */

  public static int MIN_LOGIN_NAME_LENGTH = 3;
  public static int MIN_PASSWORD_LENGTH = 5;

  private final String DEFAULT_USER_GROUP = "users";
  
  public void doPost
  (
   HttpServletRequest req,
   HttpServletResponse response
    )
       throws IOException, ServletException
  {
    
    response.setStatus(HttpServletResponse.SC_OK); 
    response.setContentType("text/html");
    
    ServletOutputStream out = response.getOutputStream();
    
    out.println( Html.header("User Addition") );
    
    String command   = req.getParameter("comm");
    String login_name= req.getParameter("user"); 
    String firstname = req.getParameter("firstname"); 
    String midname   = req.getParameter("midname");
    String lastname  = req.getParameter("lastname"); 
    String gspwd     = req.getParameter("gspwd"); 
    String confirm   = req.getParameter("confirmgspwd"); 
    
    if(login_name.equals("") || firstname.equals("") || lastname.equals("") 
       || gspwd.equals("") || confirm.equals("")) 
      out.println("<h2>ERROR: Please enter in all the fields in red!");
    else if (login_name.length() < users.MIN_LOGIN_NAME_LENGTH) 
      out.println("<h2>ERROR: Users' login names must be at least 3 digits long!");
    else if (!gspwd.equals(confirm)) 
      out.println("<h2>ERROR: Passwords do not match!");
    else if (gspwd.length() < users.MIN_PASSWORD_LENGTH) 
      out.println("<h2>ERROR: Password must be at least 5 characters long!");
    else {
      String email     = req.getParameter("email"); 
      String dept      = req.getParameter("dept"); 
      String NPAClogin = req.getParameter("login"); 
      String url       = req.getParameter("url");  
      String picurl    = req.getParameter("picurl");  
      String phone     = req.getParameter("phone"); 
      String room      = req.getParameter("room");  
      
      user u = new user(login_name, gspwd, firstname, midname, lastname,
			email, dept, NPAClogin, url, picurl, phone, room);  
      groupusers gu = new groupusers(DEFAULT_USER_GROUP, login_name);  
      
      try {
	String templateFile = req.getPathTranslated();
	Hashtable ht = new Hashtable();
	if( command.equals("new") ) {
	  update.updateData( u.insertSQL() );
	  update.updateData( gu.insertSQL() );
	  ht.put("message", "User addition comleted successfully!");
	} 
	else {
	  update.updateData( u.updateSQL() );
	  ht.put("message", "User information update comleted successfully!");
	}

	Parser.parseHtml
	  (templateFile,
	   ht,
	   out);

      } catch( Exception e ) {
	out.println("<h2>ERROR: "+e.getMessage());
      }   
      out.println("<HR>");
      
      out.println( Html.footer() );
    }
  }
  
  public void doGet(HttpServletRequest req, HttpServletResponse response)
       throws IOException, ServletException
  {
    
    response.setStatus(HttpServletResponse.SC_OK); 
    response.setContentType("text/html");
    
    ServletOutputStream out = response.getOutputStream();
    
    out.println( Html.header("User Records") );
    
    String command= req.getParameter("comm"); 
    if(command.equals("list"))
      listUsers(out);
      
    else if(command.equals("browse"))
      browseUser(out, req.getPathTranslated(), req.getParameter("user"));

    else if(command.equals("new"))
      newUser(out, req.getPathTranslated());

    else if(command.equals("delete"))
      deleteUser(out, req.getPathTranslated(), 
		 (String)(req.getParameter("user")), (String)(req.getParameter("answer")));

    out.println( Html.footer() );
    out.close();
  }
  
  void listUsers(ServletOutputStream out)
       throws IOException
  {
    try {
      out.println("<BASE TARGET=info>");
      StringBuffer output = new StringBuffer();
      user.users("login,firstname,midname,lastname","","lastname", 
		 new RecordHandler() {
	public void handleRecord(ResultSet rs, StringBuffer output) 
	  throws java.sql.SQLException {   
	  String f= rs.getString("firstname");         
	  String m= rs.getString("midname");
	  String l= rs.getString("lastname");
	  String g= rs.getString("login");
	  output.append("<tr><td><a href=\"users/Grading/users/browse.html?comm=browse&user="+
			g+"\">"+l+", "+f+" "+m+"</a></td></tr>\n");
	}
      }, output);
      out.println("<a href=\"users/Grading/users/update.html?comm=new\">Add User</a></p>");      
      out.println("<p><table border=1><tr><th>Users</th></tr>");
      out.println(output.toString()+"</table>");

    } catch( Exception e ) {
      out.println("<h2>ERROR: "+e.getMessage()+"</h2>");
    }   
  }

  void browseUser(ServletOutputStream out, String templateFile, String userlogin) 
       throws IOException
  {
    if(userlogin.length() >= users.MIN_LOGIN_NAME_LENGTH) {
      try {
	Parser.parseHtml
	  (templateFile,
	   user.getFields( userlogin ),
	   out);

      } catch(Exception e) {
	out.println("<h2>ERROR:"+e.getMessage()+"</h2>");
      }
    } else {
      out.println("<h2>ERROR:Illegal user login name '"+userlogin+"'.</h2>");
    }
  }

  void newUser(ServletOutputStream out, String templateFile) 
       throws IOException
  {
    try {
      Hashtable ht = user.getEmptyFields();
      ht.put("comm", "new");
      
      Parser.parseHtml
	(templateFile,
	 ht,
	 out);
      
    } catch(Exception e) {
      out.println("<h2>ERROR:"+e.getMessage()+"</h2>");
    }
  }

  void deleteUser
  (
   ServletOutputStream out, 
   String templateFile, 
   String userlogin,
   String answer
    ) 
       throws IOException
  {
    try {
      Hashtable ht = new Hashtable();
      if(answer == null) 
	{
	  ht.put("message", "Do you really want to delete the user <i>"+userlogin+"</i>?");
	  ht.put("actionurl", "message.html");
	  ht.put("comm","delete");
	  ht.put("user",userlogin);
	} 
      else if(answer.equals("YES")) 
	{
	  user u = new user();
	  groupusers gs = new groupusers( "", userlogin );
	  
	  u.login = userlogin;
	  update.updateData( u.deleteSQL() );
	  update.updateData( gs.deleteSQL() );
	  ht.put("message", "User information has been succesfully deleted!");
	}
      Parser.parseHtml
	(templateFile,
	 ht,
	 out);
      
    } catch(Exception e) {
      out.println("<h2>ERROR:"+e.getMessage()+"</h2>");
    }
  }
}
